#
# This is a Shiny web application. You can run the application by clicking
# the 'Run App' button above.
#
# Find out more about building applications with Shiny here:
#
#    http://shiny.rstudio.com/
#

library(shiny)

# Define UI for application that draws a figure
ui <- fluidPage(
   
   # Application title
   titlePanel("Risk of death for surgically treated breast cancer patients"),
   
   # The layout for input 
   sidebarLayout(
      sidebarPanel(
        sliderInput("time","Prediction time (days from surgery):",
                    min = 0,max = 3000,value = 1000),
        sliderInput("X","Progression time: (days from surgery)",
                    min = 0,max = 3000,value = 600),
        selectInput("er","Estrogen receptor status:",choices=c("Negative","Positive")),
        selectInput("age","Age at diagnosis:",choices=c("age<=40","40<age<=50","age>50")),
        selectInput("size","Tumor size:",choices=c("<= 2cm","> 2cm")),
        selectInput("node","Lymph nodal statu:",choices=c("absent","present")),
        selectInput("MMP","MammaPrint (the 70-gene signature):",
                    choices=c("Low","Medium(or none)","High")),
        selectInput("GGI","The gene expression grade index (GGI):",
                    choices=c("Low","Medium(or none)","High"))
      ),
      # Show a plot and numerical outputs
      mainPanel(
         plotOutput("Plot"),
         tableOutput("Num")
      )
   )
)

# Define server logic required to draw a histogram
server <- function(input, output) {
  
  library(joint.Cox)
  ### Set model parameters ###
  beta1=c(-0.15, -0.23, 0.27, 0.20, 0.19)
  beta2=c(-0.36, 0.14, 0.27, 0.17, 0.25)
  g=c(0.20, 0.39, 0.19, 0.43, 0.25) # baseline hazard coefficients
  h=c(0.05, 0.37, 0.38, 0.09, 0.00) # baseline hazard coefficients
  theta=10.7
  eta=0.067
  alpha=1
  lower=0
  upper=5000 ### less than the longest follow-up date (9108 days)
 
  Out=reactive({
    age=1*(input$age=="age<=40")+2*(input$age=="40<age<=50")+3*(input$age=="age>50")
    er=0*(input$er=="Negative")+1*(input$er=="Positive")
    size=0*(input$size=="<= 2cm")+1*(input$size=="> 2cm")
    node=0*(input$node=="absent")+1*(input$node=="present")
    GGI=-1*(input$GGI=="Low")+0*(input$GGI=="Medium(or none)")+1*(input$GGI=="High")
    MMP=-1*(input$MMP=="Low")+0*(input$MMP=="Medium(or none)")+1*(input$MMP=="High")
    
    ### Set patient-level covariates ###
    Z1=c("age"=age,"er"=er,"size"=size,"MAMMAPRINT"=MMP,"GGI"=GGI)
    Z2=c("er"=er,"node"=node,"size"=size,"MAMMAPRINT"=MMP,"GGI"=GGI)
    
    ### Set prediction time ###
    time = input$time
    X=input$X
    w_num=10
    widths=seq(0,upper-time,length=w_num)
 
    output=F.prediction(time=time, width=widths, Z1=Z1, Z2=Z2, X=X,
              beta1=beta1, beta2=beta2,eta=eta, theta=theta,
              alpha=alpha, g=g, h=h, lower, upper, Fplot=TRUE)
    colnames(output)=c("t (prediction time)","w (elapsed time from t)",
                       "X (progression time)","Probability of death")
    output
   })
  
  output$Plot=renderPlot({Out()})
  output$Num=renderTable({Out()})

}

# Run the application 
shinyApp(ui = ui, server = server)
