library(joint.Cox)
library(survival)
library(Matrix)
library(MASS)

# https://academic.oup.com/jnci/article/104/4/311/979947?login=true#supplementary-data
# import the file "jnci-JNCI-11-0924-s02.csv" in working directly 
Dat=read.csv("jnci-JNCI-11-0924-s02.csv",skip=27) # skip the first 27 rows
dat=Dat[,c("t.dmfs","e.dmfs","t.os","e.os", "dataset",
           "age","er","grade","node","size","MAMMAPRINT","GGI")]
Name=as.vector(dat[,"dataset"])
temp=c(Name=="CAL"|Name=="NKI"|Name=="TRANSBIG"|Name=="UCSF")
dat=dat[temp,] ## make a subset
dat=na.omit(dat) ## remove missing values
group=as.numeric(factor(dat[,"dataset"])) #1=CAL;2=NKI;3=TRANSBIG;4=UCSF
t.event=as.vector(dat[,"t.dmfs"]) # time-to-metastasis
event=as.vector(dat[,"e.dmfs"]) # indicator for metastasis
t.death=as.vector(dat[,"t.os"]) # overall survival
death=as.vector(dat[,"e.os"]) # indicator of death

Z=dat[,c("age","er","node","size","MAMMAPRINT","GGI")]
Z[,"MAMMAPRINT"]=mammaprint=2*c(Z[,"MAMMAPRINT"]=="HighRisk")-1
Z[,"GGI"]=mammaprint=2*c(Z[,"GGI"]=="HighRisk")-1
Z[,"size"]=as.numeric(c(Z[,"size"]>2))
Age=1*(Z[,"age"]<=40)+2*((Z[,"age"]>40)&(Z[,"age"]<=50))+3*(Z[,"age"]>50)
Z[,"age"]=Age
Z=as.matrix(Z)


####### Follow-up length #######
survfit(Surv(t.death[group==1],1-death[group==1])~1)
max(t.death[group==1])
survfit(Surv(t.death[group==2],1-death[group==2])~1)
max(t.death[group==2])
survfit(Surv(t.death[group==3],1-death[group==3])~1)
max(t.death[group==3])
survfit(Surv(t.death[group==4],1-death[group==4])~1)
max(t.death[group==4])
survfit(Surv(t.death,1-death)~1)
max(t.death)

##### Marginal significance analysis ######
coxph(Surv(t.event,event)~Z[,"age"]) 
coxph(Surv(t.event,event)~Z[,"er"]) 
coxph(Surv(t.event,event)~Z[,"node"]) ## Not significant
coxph(Surv(t.event,event)~Z[,"size"])
coxph(Surv(t.event,event)~Z[,"MAMMAPRINT"])
coxph(Surv(t.event,event)~Z[,"GGI"])

coxph(Surv(t.death,death)~Z[,"age"]) ## Not significant
coxph(Surv(t.death,death)~Z[,"er"])
coxph(Surv(t.death,death)~Z[,"node"]) 
coxph(Surv(t.death,death)~Z[,"size"])
coxph(Surv(t.death,death)~Z[,"MAMMAPRINT"])
coxph(Surv(t.death,death)~Z[,"GGI"])      

##### fit a Joint model #####
z1=Z[,c("age","er","size","MAMMAPRINT","GGI")]
z2=Z[,c("er","node","size","MAMMAPRINT","GGI")]

alpha=1
grid=seq(10,1e+18,length=30) 

set.seed(1)
Res=jointCox.reg(t.event=t.event, event=event, t.death=t.death, death=death,
                 Z1=z1, Z2=z2, group=group, alpha=alpha,Randomize_num = 1,
                 kappa1=grid, kappa2=grid, LCV.plot=TRUE, Adj=500,
                 convergence.par = TRUE)

par(mfrow=c(1,1)) ## Reset the figure space

### Set model parameters ###
beta1=Res$beta1[1:5];round(beta1,2)
beta2=Res$beta2[1:5];round(beta2,2)
g=Res$g;round(g,2) # baseline hazard coefficients
h=Res$h;round(h,2) # baseline hazard coefficients
theta=Res$theta[1];round(theta,3)
eta=Res$eta[1];round(eta,3)
alpha=1
lower=0
upper=9108

### Set patient-level covariates (Patient 1) ###
Z1=c("age"=2,"er"=1,"size"=1,"MAMMAPRINT"=1,"GGI"=1)
Z2=c("er"=1,"node"=1,"size"=1,"MAMMAPRINT"=1,"GGI"=1)

Pt1=(z1[,"age"]==Z1["age"])&(z1[,"er"]==Z1["er"])&
  (z2[,"node"]==Z2["node"])&(z1[,"size"]==Z1["size"])&
  (z1[,"MAMMAPRINT"]==Z1["MAMMAPRINT"])&(z1[,"GGI"]==Z1["GGI"])
dat[Pt1,] ###


### Set prediction parameters ###
time=1000 
w_num=20
widths=seq(0,upper-time,length=w_num)
round(widths,0)
X=2000
par(mfrow=c(1,1)) ## Reset the figure space
M=500 # the number of Monte Carlo simulations
p1=length(beta1)
p2=length(beta2)
mu=Res$convergence.parameters$log_estimate
Sigma=nearPD(Res$convergence.parameters$log_var)$mat

##### F(t,t+w|X>t) #####
F.prediction(time=time, width=widths, Z1=Z1, Z2=Z2, X=X,
             beta1=beta1, beta2=beta2,eta=eta, theta=theta,
             alpha=alpha, g=g, h=h, lower, upper, Fplot=TRUE)

##### 95%CI by Monte Carlo simulation #######

F.m=matrix(0,w_num,M)

for(m in 1:M){
  set.seed(m)
  phi.m=mvrnorm(1, mu = mu, Sigma = Sigma)
  g.m=pmin( exp( phi.m[1:5] ), 100*Res$g ) ## M-spline coefficients ##
  h.m=pmin( exp( phi.m[6:10] ), 100*Res$h ) ## M-spline coefficients ##
  eta.m=exp( phi.m[11] )
  theta.m=exp( phi.m[12])
  beta1.m=phi.m[(12+1):(12+p1)]
  beta2.m=phi.m[(12+p1+1):(12+p1+p2)]
  
  F.m[,m]=F.prediction(time=time,width=widths,Z1=Z1,Z2=Z2,X=X,
                       beta1=beta1.m,beta2=beta2.m,eta=eta.m,theta=theta.m,alpha=alpha,
                       g=g.m,h=h.m,lower,upper,Fplot=FALSE)[,"F"]
}

F.lower=apply(F.m,MARGIN=1,function(x) quantile(x,probs=0.025))
F.upper=apply(F.m,MARGIN=1,function(x) quantile(x,probs=0.975))
points(time+widths,F.lower,col="blue",type="l",lty="dotted")
points(time+widths,F.upper,col="blue",type="l",lty="dotted")

F.diff=F.upper-F.lower
F.char=as.character(round(F.diff,2))
text(time+widths[c(5,10,15)],F.lower[c(5,10,15)]-0.1,
     F.char[c(5,10,15)])

segments(time+widths[c(5,10,15)],F.lower[c(5,10,15)],
         time+widths[c(5,10,15)],F.upper[c(5,10,15)])
points(time+widths[c(5,10,15)],F.upper[c(5,10,15)],pch=24)
points(time+widths[c(5,10,15)],F.lower[c(5,10,15)],pch=25)



##### F(t,t+w|X=300) #####
X=300
F_X=F.prediction(time=time, width=widths, Z1=Z1, Z2=Z2, X=X,
             beta1=beta1, beta2=beta2,eta=eta, theta=theta,
             alpha=alpha, g=g, h=h, lower, upper, Fplot=FALSE)[,"F"]

##### 95%CI by Monte Carlo simulation #######
F.m=matrix(0,w_num,M)

for(m in 1:M){
  set.seed(m)
  phi.m=mvrnorm(1, mu = mu, Sigma = Sigma)
  g.m=pmin( exp( phi.m[1:5] ), 100*Res$g ) ## M-spline coefficients ##
  h.m=pmin( exp( phi.m[6:10] ), 100*Res$h ) ## M-spline coefficients ##
  eta.m=exp( phi.m[11] )
  theta.m=exp( phi.m[12])
  beta1.m=phi.m[(12+1):(12+p1)]
  beta2.m=phi.m[(12+p1+1):(12+p1+p2)]
  
  F.m[,m]=F.prediction(time=time,width=widths,Z1=Z1,Z2=Z2,X=X,
                       beta1=beta1.m,beta2=beta2.m,eta=eta.m,theta=theta.m,alpha=alpha,
                       g=g.m,h=h.m,lower,upper,Fplot=FALSE)[,"F"]
}

F.lower=apply(F.m,MARGIN=1,function(x) quantile(x,probs=0.025))
F.upper=apply(F.m,MARGIN=1,function(x) quantile(x,probs=0.975))
points(time+widths,F_X,col="red",type="l",lwd=2,lty="solid")
points(time+widths,F.lower,col="red",type="l",lty="dotted")
points(time+widths,F.upper,col="red",type="l",lty="dotted")

F.diff=F.upper-F.lower
F.char=as.character(round(F.diff,2))
text(time+widths[c(5,10,15)],F.lower[c(5,10,15)]-0.05,
     F.char[c(5,10,15)])

segments(time+widths[c(5,10,15)],F.lower[c(5,10,15)],
         time+widths[c(5,10,15)],F.upper[c(5,10,15)])
points(time+widths[c(5,10,15)],F.upper[c(5,10,15)],pch=24)
points(time+widths[c(5,10,15)],F.lower[c(5,10,15)],pch=25)


AA=c("Metastasis at 300 days",
     "No metastasis by 1000 days","95% CI")
BB=c("solid","solid","dotted")
CC=c("red","blue","black")
legend("bottomright",AA,lwd=c(2,2,1),merge = TRUE,lty=BB,col=CC)

########### Prediction error ############
w_num=50
widths=seq(0,upper-time,length=w_num)

### Censoring survival function ###
cen_KM=survfit(Surv(t.death, 1-death)~1)
c.sort=summary(cen_KM)$time
cen_SKM=summary(cen_KM)$surv
cen_SKM=c(1,cen_SKM[-length(cen_SKM)])
G_time=cen_SKM[ max(sum(c.sort<=time),1) ]
G_widths=numeric(w_num)
for(i in 1:w_num){
  G_widths[i]=cen_SKM[max(sum(c.sort<=time+widths[i]),1)]
}

F_widths=F.KM(time=time,widths=widths,t.death=t.death,death=death)[,"F"]

####### Compute F ########
F_joint=w=I=NULL
n=length(t.event)
for(i in 1:n){
  if( t.death[i]>time){ 
    F_res=F.prediction(time=time,width=widths,Z1=z1[i,],Z2=z2[i,],
                       X=t.event[i],beta1=beta1,beta2=beta2,eta=eta,theta=theta,
                       alpha=alpha,g=g,h=h,lower,upper,Fplot=FALSE)[,"F"]
    F_joint=rbind(F_joint,F_res)
    G_death=cen_SKM[max(sum(c.sort<=t.death[i]),1)]
    w_at=(t.death[i]<=time+widths)*death[i]*G_time/G_death
    w_at=w_at+(t.death[i]>time+widths)*G_time/G_widths
    w=rbind(w,w_at)
    I=rbind(I,as.numeric(t.death[i]>time+widths))
  }
}

Err_joint=colMeans( w*(I-(1-F_joint))*(I-(1-F_joint)) )
F_KM=matrix(F_widths,nrow(I),ncol(I),byrow=TRUE)
Err_KM=colMeans( w*((I-(1-F_KM))*(I-(1-F_KM))) )


###### calibration plot by the joint model ######
par(mfrow=c(1,2))

Pred_joint=colMeans( (w*(1-F_joint))[,-1] )
Obs_joint=colMeans( (w*I)[,-1])

plot(Obs_joint,Pred_joint,xlim=c(0,1),ylim=c(0,1),col="red",
     main="Joint model",xlab="Pred (w)",ylab="Obs (w)")
lines(x = c(0,1), y = c(0,1))


###### calibration plot by the KM ######
Pred_KM=colMeans( (w*(1-F_KM))[,-1] )
Obs_KM=colMeans( (w*I)[,-1])

plot(Obs_KM,Pred_KM,xlim=c(0,1),ylim=c(0,1),col="red",
     main="Kaplan-Meier",xlab="Pred (w)",ylab="Obs (w)")
lines(x = c(0,1), y = c(0,1))


######## Plot prediction errors #####
B=1000 # the number of bootstrap resamples
F_boot=matrix(0,B,w_num)

for(b in 1:B){
  boot=sample(1:nrow(I),replace=TRUE)
  F_boot[b,]=colMeans( (w*(I-(1-F_joint))*(I-(1-F_joint)))[boot,] )
}

F_lower=apply(F_boot, 2, sort)[0.025*B,]
F_upper=apply(F_boot, 2, sort)[0.975*B,]


par(mfrow=c(1,2)) ## Reset the figure space
plot(time+widths, Err_KM, xlim = c(time/1.2, upper),ylim=c(0,0.25),
     type="l",lwd=2,xlab="t+w (days)",lty="dashed",ylab="Err ( t, t+w )")
abline(h = 0)
abline(v = time, col = "gray")
text(time-50,0.01,"t")

points(time+widths,F_upper,type="l",lwd=1,col="red",lty="dashed" )
points(time+widths,F_lower,type="l",lwd=1,col="red",lty="dashed" )
points(time+widths,Err_joint,type="l",lwd=2,col="red",lty="solid" )

AA=c("Kaplan-Meier","Joint model","95% CI")
BB=c("dashed","solid","dotted")
CC=c("black","red","red")
legend("bottom",AA,lwd=c(2,3,1),merge = TRUE,lty=BB,col=CC)


####### compute c-index ######
C=C_upper=C_lower=numeric(w_num)
for(i in 2:w_num){
  t.death_w=pmin(t.death[t.death>time],time+widths[i])
  death_w=(death[t.death>time])*(t.death[t.death>time]<=time+widths[i])
  C_res=concordance(Surv(t.death_w,death_w)~F_joint[,i])
  C[i]=1-C_res$concordance
  SE=sqrt(C_res$var)
  C_upper[i]=1-C_res$concordance+1.96*SE
  C_lower[i]=1-C_res$concordance-1.96*SE
}

plot(time+widths[-1], C[-1], xlim = c(time/1.2, upper),ylim=c(0,1),
     type="l",lwd=2,xlab="t+w (days)",ylab="c-index",col="red")
abline(h = 0.5,col="blue")
abline(v = time, col = "gray")
points(time+widths[-1],C_upper[-1],type="l",lwd=1,col="red",lty="dashed" )
points(time+widths[-1],C_lower[-1],type="l",lwd=1,col="red",lty="dashed" )

AA=c("c-index","95% CI")
BB=c("solid","dotted")
CC=c("red","red")
legend("bottom",AA,lwd=c(3,1),merge = TRUE,lty=BB,col=CC)


